(*Ast.ml file for Nifty50 by Aamir Jahan, COMS W4115, Fall 2014*)

(* declaring all the binary and unary operator type in Nifty50 *)
type binop = Add | Sub | Mult | Div | Equal | Neq | Less | Leq | Greater | Geq | Bsl | Bsr | And | Or 
| Xor | Nand | Nor 

type unop = Not

(* declare modules for the language *)
type modules = Variable | Class | Function | Interrupt

(* declare all data types *)
type datatypes = Array of datatypes * string | Int | Double | String | Port | Void |NewType of string

(* declaring type expression; an expr can be:- *)
(* int, binary, hexadecimal, port, string, binop, assignment, call *)
type expr =
    ILiteral of int
	| FLiteral of float
	| SLiteral of string
	| Variable of datatypes
  | Id of string
	| Unop of unop * expr
  | Binop of expr * binop * expr
  | Assign of string * expr
	| CastType of datatypes * expr			(* check this later *)
	| Address of string * expr
	| GetAddress of string
	| ArrayIndex of string * string
	| Method of expr * expr
	| Swap of string
	| Signal of string * expr
	| Map of string * string * expr * expr
	| Call of string * expr list
	| MethodCall of string * string * expr list
  | Noexpr

(* statement can be a block, expression, if, for, while *)
type stmt =
    Block of stmt list
  | Expr of expr
  | Return of expr
  | If of expr * stmt * stmt
  | For of expr * expr * expr * stmt list 
  | While of expr * stmt

(* declaring variable data types *)
type variable_declaration = {
	vtype: datatypes;
	vname: string;
	}
(* function declarations are fname, formals, locals, body *)
type function_declaration = {
	rtype : datatypes;
  fname : string;
 	formals : variable_declaration list;
 	locals : variable_declaration list;
 	body : stmt list;
  }

(* declaring interrupt type *)
type interrupt_declaration = {
	iname : string;
	ilocals : variable_declaration list;
	ibody : stmt list;
}

type type_declaration = {
	ytype: datatypes;
	yname: string;
	yproperties: variable_declaration list;
	yfunctions: function_declaration list;
}

let first = fun (a,b,c,d) -> a
let second = fun (a,b,c,d) -> b
let third = fun (a,b,c,d) -> c
let fourth = fun (a,b,c,d) -> d



(* program returns a list of strings and func_decl list *)
type program = function_declaration list * type_declaration list 
								* interrupt_declaration list * variable_declaration list

(* End of File*)